#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGraphicsView>
#include <QToolBar>
#include "scene_cliquable.h"
#include <QAction>
#include <QGraphicsTextItem>
#include <QGraphicsLineItem>
#include <QGraphicsItemGroup>
#include <QTimer>

#include "ui_mainwindow.h"
#include "boutonled.h"
#include "qled.h"

#include "qextserialport.h"



struct Aperture
{
    char forme; // = 'R'(Rectangle) ou 'C'(Cercle) // attention :  ne pas confondre avec R comme ronde...!
    QPoint WH;
};



struct Element
{
    quint16 num_origine; // numéro d'ordre dans la liste gerber d'origine
    quint16 nouveau_num; // numéro d'ordre dans la liste gerber triée
    QString texte_affichable; // reprend les données sous forme texte pour pouvoir les afficher dans les tables
    char sorte; // = 'M' (ajout aperture) ou 'S'(segment)  ou ='P' (pastille)
    qint16 num;
    qint8 num_aperture;
    QPoint A;
    QPoint B;
    QPoint M; // milieu

    QColor couleur;
    int clique; // cliqué. = 0 ou 1
};

/*
struct Distance
{
    quint16 num_origine;
    qint16 ds; // distance à un élément donné
};
*/


//class QAction;


class MainWindow : public QMainWindow, public Ui::MainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    
    SceneCliquable *scene1;

private slots:

  //  qreal square(qreal r1);

    void InitTable3();
    void InitTable4();
    void InitTable5();

    void lecture_fichier_init();
    void enregistrer_fichier_init();

    void efface_buffer();
    void onReadyRead();

    qint8 Connexion(qint8 cnx);
    void effacer_ecran();
    void effacer_pointage();


    void affiche_liste_elements();
    void affiche_liste_elements_tries();

    int recharger_fichier();
    int lire_fichier_gerber(QString nom_fichier1);
    int analyse_1ligne_fichier();
    void analyse_fichier_gerber();
    void creation_liste_elements();
    void creation_liste_segments();
    void creation_liste_trous();

    void raccourcir_segment(Element *segment_i, quint16 ls, char bout);
    void inverser_element(Element *EL);

//    void compte_pistes_nulles();
//    void supprimer_pistes_nulles();

    void Timer1_clic();
    void Timer2_clic();
    void Timer3_clic();

    void flash_ligne_suivante();

    int on_Btn_envoi_clicked();

    void record_fichier_log(QString dossier1, QString fichier_out);

    void on_Btn_RAZ_pos_clicked();
    void on_Btn_gotoHome_clicked();
    void on_Btn_gotoSafePos_clicked();
    void on_Btn_GOTO_clicked();

 //   void on_Btn_TRACER_AUTO_clicked();
 //   void on_Btn_trace1ligne_clicked();
    int Envoi_une_ligne();

    void pointer(int i);
    void tout_decliquer();

    void tracer_mire_origine();
    void tracer_segment(QPoint A, QPoint B, qreal largeur, QColor couleur_i, QGraphicsItemGroup *groupe_i);
    void tracer_pastille_ronde(QPoint A, QPoint B, QColor couleur_i, QGraphicsItemGroup *groupe_i);
    void tracer_pastille_rectangulaire(QPoint A, QPoint B, QColor couleur_i, QGraphicsItemGroup *groupe_i);
    void tracer_trou(QPoint A, quint16 diametre);

    void tracer_texte(int x, int y, QString texte_i);
    int dessine_1ligne_liste_triee(int i);
    void dessiner_liste_elements_tries();
    int detecte_element_pointe(QPoint P);
    quint16 detecte_element_proche();

    void classer_pastilles();
    void classer_le_reste();
    void classer_auto();
    void creuser_pastilles();


    void inverser_miroir_elements();

    void trace_liste_apertures();
    QString mise_en_forme_ligne_pour_envoi(QString s_i);
    void envoi_vers_Atmega(QString s1);

 //   void on_Btn_ralentir_clicked();
    void on_Btn_pastille_clicked();
    int on_actionOuvrir_triggered();
    void on_actionRecharger_triggered();
    void on_actionConnexion_triggered(bool checked);

    void on_Btn_stop_clicked();
//    void on_Btn_stop3_clicked();

    void on_Btn_R_clicked();
    void on_Btn_Xplus_clicked();
    void on_Btn_Xmoins_clicked();
    void on_Btn_Yplus_clicked();
    void on_Btn_Ymoins_clicked();
    void on_Btn_Xplus1mm_clicked();
    void on_Btn_Yplus1mm_clicked();
    void on_Btn_Xplus10mm_clicked();
    void on_Btn_Yplus10mm_clicked();
    void on_Btn_Xmoins10mm_clicked();
    void on_Btn_Xmoins1mm_clicked();
    void on_Btn_Ymoins1mm_clicked();
    void on_Btn_Ymoins10mm_clicked();
 //   void on_Btn_Zmoins1mm_clicked();
//    void on_Btn_Zplus1mm_clicked();
//    void on_Btn_Zmoins5mm_clicked();
//    void on_Btn_Zplus5mm_clicked();
  //  void on_Btn_go_debut_fichier_clicked();
    void on_Btn_effacer_clicked();
 //   void on_BoutonLed1_toggled(bool etat);

    void on_spinBox_zoom2_valueChanged(double arg1);
    void on_Btn_trace_apertures_clicked();
    void on_Btn_redessiner_clicked();
    int on_Btn_tout_flasher_clicked();
    void on_BoutonLed1_toggled(bool etat);
    void on_BoutonLed2_toggled(bool etat);
    void on_BoutonLed3_toggled(bool etat);
    void on_BoutonLed4_toggled(bool etat);

    void on_Btn_stop_flash_clicked();

//    void on_Btn_lever_plume_clicked();
    void on_Btn_pastille_2_clicked();
//    void on_Btn_affleure_plume_clicked();
//    void on_Btn_touche_plume_clicked();

  //  void on_tableWidget_4_cellClicked(int row, int column);
    void onReceptPos(int px, int py);
    void onReceptMove(int dx, int dy);

 //   void mousePressEvent(QMouseEvent *event);
    int on_tableWidget_4_cellClicked(int row, int column);
    void on_tableWidget_5_cellClicked(int row, int column);
    void on_Btn_classe_clicked();
    void on_Btn_aff5_ralenti_clicked();
    void on_Btn_declique_tout_clicked();
    void on_actionAide_triggered();
    void on_Btn_stop_ralenti_clicked();
    void on_Btn_piste_X_clicked();
    void on_Btn_MIRE_clicked();
    void on_Btn_TEST_clicked();
    void on_Btn_rot_n_pas_X_clicked();
    void on_Btn_rot_n_pas_Y_clicked();
    void on_Btn_PETIT_CARRE_clicked();
    void on_Btn_GOTO_20x20_clicked();
    void on_Btn_RAZ_flash_clicked();
    void publier(QString msg_i);

    void annonce_vocale(int n_i);
//    void on_Btn_creuser_pastilles_clicked();

    void on_Btn_lum_laser_clicked();
    void on_Btn_Z_laser_clicked();
    void on_Btn_correction_clicked();
    void on_Btn_lire_ERR_clicked();
    void on_Btn_GOTO_10x10_clicked();
    void on_Btn_gotoHome_2_clicked();
    void on_Btn_help_LW4_clicked();
    void on_checkBox_miroir_toggled(bool checked);
    void on_checkBox_creuser_toggled(bool checked);
    void on_actionttyUSB0_triggered();
    void on_actionttyUSB1_triggered();
    void on_actionttyACM0_triggered();
    void on_actionttyACM1_triggered();
    void on_Btn_LST_AP_clicked();
    void on_actionAffichage_tendu_triggered(bool checked);
    void on_actionttyACM2_triggered();
    void on_actionttyACM3_triggered();
    void on_actionttyACM4_triggered();
    void on_Btn_close_Tx1_clicked();

    void on_Btn_save_log_clicked();

 //   void mouseMoveEvent(QMouseEvent *event);


private:

    void createActions();
    void createConnections();
    void createToolBar();

    QAction* lineAction;
    QAction* selectAction;
    QActionGroup *actionGroup;
    QToolBar* drawingToolBar;


    QextSerialPort *portUSB;


    QGraphicsItemGroup *groupe_trace;
    QGraphicsItemGroup *groupe_pastilles_faites;
    QGraphicsItemGroup *groupe_segments_faits;
    QGraphicsItemGroup *groupe_pointage;
    QGraphicsItemGroup *groupe_trous;

    //QGraphicsLineItem *ligne1;
   // QGraphicsEllipseItem *ellipse;
    //QGraphicsRectItem *rectangle;
    //QGraphicsLineItem *segment_trace;
   // QGraphicsLineItem *segment_pointage;
    //QGraphicsTextItem *gtexte;

    //QGraphicsTextItem *texte_frq;

    QLed *Led0;
    QLed *Led1;
    QLed *Led2;
    QLed *Led3;
    QLed *Led4;
 //   QLed *Led5;

    QBoutonLed *BoutonLed1;
    QBoutonLed *BoutonLed2;
    QBoutonLed *BoutonLed3;
    QBoutonLed *BoutonLed4;

    QTimer *Timer1; // pour la lecture au ralenti du fichier gerber
    QTimer *Timer2; // pour le tracé au ralenti des éléments triés par ordre de proximités
    QTimer *Timer3; // pour faire clignoter des affichages (par exemple LED fon de course)

  //  QAction *centrerAct;



};




#endif // MAINWINDOW_H
